<?php
class Sprypay_Sprypay_Model_Payment extends Mage_Payment_Model_Method_Abstract
{
    protected $_isGateway               = true;
    protected $_canUseInternal          = false;
    protected $_canUseForMultishipping  = false;

    protected $_code = 'sprypay';

    private $_order;
    private $_ipnData = array();
    private $_ipnValidator;

    /**
     * Возвращает URL для перенаправления покупателя после создания заказа.
     *
     * @access public
     * @return string
     */
    public function getOrderPlaceRedirectUrl()
    {
        return Mage::getUrl('sprypay/index/redirect', array('_secure' => true));
    }

    /**
     * Возвращает данные для перенаправления на Sprypay.
     *
     * @access public
     * @return array
     */
    public function getRedirectFormFields()
    {
        $result = array();

        $order = $this->getOrder();
        if (!$order->getId()) {
            return $result;
        }

        $result['shopId']    = Mage::getStoreConfig('payment/sprypay/sprypay_shop_id');
        $result['userEmail'] = $order->getCustomerEmail();
        $result['amount']    = $order->getGrandTotal();
        $result['currency']  = strtolower($order->getOrderCurrencyCode());
        $result['paymentId'] = $order->getIncrementId();
        $result['purpose']   = 'Order id #'.$result['paymentId'];
        $result['src'] = '1';
        $language = strtolower(Mage::app()->getLocale()->getLocaleCode());
        $result['language']  = ($language == 'ru') ? $language : 'en';

        return $result;
    }

    /**
     * Валидация оповещения о платеже от Sprypay.
     *
     * @access public
     * @return mixed
     */
    public function validateIpn($request)
    {
        $validator = $this->getIpnValidator();

        try {
            $validator->spParams = $request->getParams();
        } catch (SprypayException $e) {
            return $e->getMessage();
        }

        $ipnData = $validator->getNotificationData();
        $this->_ipnData = $ipnData;
        $paymentId = (int) $ipnData['shopPaymentId'];

        // Поиск заказа в системе
        $order = $this->loadOrderById($paymentId);
        $this->_order = $order;
        if (!$order)
            return "Order with id \"$paymentId\" not found";

        // Проверка идентификатора магазина
        if (!$validator->validateShopId(Mage::getStoreConfig('payment/sprypay/sprypay_shop_id')))
            return "Invalid shop id: ".$ipnData['shopId'];

        // Проверка суммы платежа
        if (!$validator->validateAmount($order->getGrandTotal()))
            return "Invalid amount: ".$ipnData['amount'];

        // Проверка валюты платежа
        if (!$validator->validateCurrency($order->getOrderCurrencyCode()))
            return "Invalid currency: ".$ipnData['currency'];

        // Проверка плательщика комиссии
       // if (!$validator->validateFeePayer(Mage::getStoreConfig('payment/sprypay/sprypay_fee_payer')))
        //    return "Invalid fee payer: ".$ipnData['takeFeeFrom'];

        // Проверка контрольной суммы
        $secret = Mage::helper('core')->decrypt(Mage::getStoreConfig('payment/sprypay/sprypay_secret'));
        if (!$validator->validateControlSum($secret))
            return "Invalid control sum: ".$ipnData['hashString'];

        return true;
    }

    /**
     * Обновляет статус заказа после поступления оплаты.
     *
     * @access public
     * @return bool
     */
    public function updateOrderState()
    {
        $ipnData = $this->getIpnData();
        $order = $this->getOrder();
        $message = 'Sprypay payment '.$ipnData['paymentId'].' ('.$ipnData['balanceAmount'].' '.$ipnData['balanceCurrency'].') was enrolled to Sprypay balance in '.$ipnData['enrollDateTime'];
        $order->setState(Mage_Sales_Model_Order::STATE_PROCESSING, Mage_Sales_Model_Order::STATE_PROCESSING, $message);
        $order->save();
        return true;
    }

    /**
     * Подтверждение для Sprypay.
     *
     * @access public
     * @return string
     */
    public function confirmNotification()
    {
        return $this->getIpnValidator()->confirmNotification();
    }

    /**
     * Возвращает текущий заказ.
     *
     * @access public
     * @return mixed
     */
    public function getOrder()
    {
        if (!$this->_order) {
            $session = Mage::getSingleton('checkout/session');
            $this->_order = $this->loadOrderById($session->getLastRealOrderId());
        }
        return $this->_order;
    }

    /**
     * Загружает заказ по идентификатору.
     *
     * @param int $orderId
     * @access private
     * @return mixed
     */
    private function loadOrderById($orderId)
    {
        return Mage::getModel('sales/order')->loadByIncrementId($orderId);
    }

    /**
     * Возвращает данные пришедшие в оповещении о платеже.
     *
     * @access public
     * @return array
     */
    public function getIpnData()
    {
        return $this->_ipnData;
    }

    /**
     * Возвращает объект для валидации оповещения о платеже.
     *
     * @access private
     * @return SprypayNotificationValidator
     */
    private function getIpnValidator()
    {
        if (!$this->_ipnValidator) {
            $file = Mage::getBaseDir().DS.'lib'.DS.'sprypaylib'.DS.'SprypayNotificationValidator.php';
            if (file_exists($file))
                require_once($file);
            else
                throw new Exception('Sprypaylib: class '.$className.' not found');
            $this->_ipnValidator = new SprypayNotificationValidator();
        }
        return $this->_ipnValidator;
    }
}
